from rest_framework import exceptions
import collections.abc


def update(d, u):
    for k, v in u.items():
        if isinstance(v, collections.abc.Mapping):
            d[k] = update(d.get(k, {}), v)
        else:
            d[k] = v
    return d


class Config:
    required = ['email', 'name', 'phone']

    def __init__(self, config: dict):
        self.config = config

    def check_config(self, config):
        for field in self.required:
            if field not in config.keys():
                raise exceptions.ValidationError

    def update_config(self, **kwargs):
        config = update(self.config, kwargs)
        self.check_config(config)
        return config



from django.db import models
from revauth.settings import api_settings
from django.conf import settings
from django.contrib import admin
from django.contrib.auth.models import User


class BaseProfile(models.Model):
    PROVIDERS = (('default', 'default'), ('facebook', 'facebook'), ('google', 'google'), ('apple', 'apple'))
    user = models.OneToOneField(User, on_delete=models.CASCADE, related_name='profile')
    name = models.CharField(max_length=128, null=True, blank=True)
    email = models.EmailField(null=api_settings.DEFAULT_IDENTITY != 'email', blank=api_settings.DEFAULT_IDENTITY != 'email')
    phone = models.CharField(max_length=128, null=api_settings.DEFAULT_IDENTITY != 'phone', blank=api_settings.DEFAULT_IDENTITY != 'phone')
    is_staff = models.BooleanField(default=False)
    avatar = models.URLField(blank=True, null=True)
    provider = models.CharField(max_length=128, choices=PROVIDERS, default='default')
    social_id = models.CharField(max_length=256, null=True, blank=True)

    class Meta:
        abstract = True


class UserProfile(BaseProfile):
    class Meta:
        abstract = 'revauth' not in settings.INSTALLED_APPS
        app_label = 'revauth'


if not UserProfile._meta.abstract:
    admin.site.register(UserProfile)



from rest_framework import exceptions


class APIError(exceptions.APIException):
    status_code = 500
    detail = None
    code = None

    def __init__(self, detail=None, code=None):
        if detail:
            self.detail = detail
        if code:
            self.code = code


class UserExistsError(APIError):
    status_code = 400
    detail = 'user is exists'
    code = 'user_exists'


class UserNotFoundError(APIError):
    status_code = 404
    detail = 'user not found'
    code = 'user_not_found'


class InvalidCodeError(APIError):
    status_code = 400
    detail = 'this code is invalid'
    code = 'invalid_code'


class JWTDecodError(APIError):
    status_code = 400
    detail = 'jwt decode error'
    code = 'invalid_jwt'


class APIKeyError(APIError):
    status_code = 403
    detail = 'api key not valid'
    code = 'api_key_error'



from django.urls import path
from revauth import views

urlpatterns = [
    path('google', views.GoogleLogin.as_view(), name='auth-google'),
    path('facebook', views.FacebookLogin.as_view(), name='auth-facebook'),
    path('apple', views.AppleLogin.as_view(), name='auth-apple'),
    path('validation/request', views.ValidationView.as_view(), name='auth-validation'),
    path('user/register', views.UserRegister.as_view(), name='auth-register'),
    path('user/login', views.UserLogin.as_view(), name='auth-login'),
    path('user/refresh', views.RefreshProfile.as_view(), name='auth-refresh'),
]



from django.conf import settings
from django.http import HttpResponse
from rest_framework import response
from django.contrib.auth.models import User
from revauth.settings import api_settings
import json


class SocialSDK:
    social_type = None
    handler_class = api_settings.DEFAULT_HANDLER_CLASS

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)

    def get_provider_class(self):
        from .providers import FacebookProvider, GoogleProvider, AppleProvider
        if self.social_type == 'facebook':
            return FacebookProvider
        elif self.social_type == 'google':
            return GoogleProvider
        elif self.social_type == 'apple':
            return AppleProvider

    def get_provider(self):
        provider_class = self.get_provider_class()
        return provider_class()

    def login(self, token, **kwargs):
        provider = self.get_provider()
        profile = provider.login(token, **kwargs)
        handler = self.handler_class()
        result = handler.on_login_success(profile)
        return response.Response(result, 200)



import jwt
from jwcrypto.jwt import JWK
from django.conf import settings
from revauth.settings import api_settings
import requests
import json


class BaseCryptor:
    pub_key = settings.PUB_KEY
    audience = settings.CLIENT_ID
    API_KEY = settings.CLIENT_SECRET
    DEBUG = settings.DEBUG
    VERSION = api_settings.AUTH_VERSION

    @property
    def api_host(self):
        if self.DEBUG:
            return 'https://auth-stg.revtel-api.com/' + self.VERSION
        return 'https://auth.revtel-api.com/' + self.VERSION

    def decode(self, token):
        return jwt.decode(token, self.pub_key, audience=self.audience)

    def _post(self, path, data):
        headers = {'x-api-key': self.API_KEY, 'Content-Type': 'application/json'}
        resp = requests.post(f'{self.api_host}{path}', json=data, headers=headers)
        return resp


class JWTCryptor(BaseCryptor):
    def decode_register(self, token):
        decoded = self.decode(token)
        if decoded['typ'] != 'register':
            raise
        return decoded

    def encode(self, data):
        path = '/jwt/server/encode'
        resp = self._post(path, data)
        return resp.json()


class GoogleCryptor(BaseCryptor):
    audience = api_settings.GOOGLE_CLIENT_ID

    def __init__(self, headers):
        kid = headers['kid']
        keys = requests.get('https://www.googleapis.com/oauth2/v3/certs').json()['keys']
        key = [k for k in keys if k['kid'] == kid][0]
        key = JWK.from_json(json.dumps(key))
        self.pub_key = key.export_to_pem()


class AppleCryptor(BaseCryptor):
    audience = api_settings.APPLE_CLIENT_ID

    def __init__(self, headers):
        kid = headers['kid']
        keys = requests.get('https://appleid.apple.com/auth/keys').json()['keys']
        key = [k for k in keys if k['kid'] == kid][0]
        key = JWK.from_json(json.dumps(key))
        self.pub_key = key.export_to_pem()



from django.conf import settings
from django.utils.module_loading import import_string


DEFAULTS = {
    # Base API policies
    'DEBUG': True,
    'AUTH_VERSION': 'v2',
    'DEFAULT_IDENTITY': 'email',
    'DEFAULT_PROFILE_CLASS': 'revauth.models.UserProfile',
    'DEFAULT_BASE_PATH': 'auth',
    'DEFAULT_HANDLER_CLASS': 'revauth.handlers.DefaultHandler',
    'DEFAULT_REGISTER_SERIALIZER': 'revauth.serializers.RegisterSerializer',
    'DEFAULT_VALIDATION_ISSUER': 'register',
    'GOOGLE_CLIENT_ID': '782455260604-925fnvgriavsgqs89edlecqcgfee8d49.apps.googleusercontent.com',
    'APPLE_CLIENT_ID': 'fake'
}
# List of settings that may be in string import notation.
IMPORT_STRINGS = [
    'DEFAULT_PROFILE_CLASS', 'DEFAULT_HANDLER_CLASS', 'DEFAULT_REGISTER_SERIALIZER'
]


def perform_import(val, setting_name):
    """
    If the given setting is a string import notation,
    then perform the necessary import or imports.
    """
    if val is None:
        return None
    elif isinstance(val, str):
        return import_from_string(val, setting_name)
    elif isinstance(val, (list, tuple)):
        return [import_from_string(item, setting_name) for item in val]
    return val


def import_from_string(val, setting_name):
    """
    Attempt to import a class from a string representation.
    """
    try:
        return import_string(val)
    except ImportError as e:
        msg = "Could not import '%s' for API setting '%s'. %s: %s." % (val, setting_name, e.__class__.__name__, e)
        raise ImportError(msg)


class APISettings:
    def __init__(self, defaults=None, import_strings=None):
        self.defaults = defaults or DEFAULTS
        self.import_strings = import_strings or IMPORT_STRINGS

    @ property
    def user_settings(self):
        return getattr(settings, 'REVAUTH', {})

    def __getattr__(self, attr):
        try:
            # Check if present in user settings
            val = self.user_settings[attr]
        except KeyError:
            # Fall back to defaults
            val = self.defaults[attr]

        # Coerce import strings into classes
        if attr in self.import_strings:
            val = perform_import(val, attr)

        # Cache the result
        setattr(self, attr, val)
        return val


api_settings = APISettings(DEFAULTS, IMPORT_STRINGS)
